//	TorusGamesRectangle.c
//
//	Makes, binds and draws a Vertex Array Object for a rectangle.
//
//	© 2010 by Jeff Weeks
//	See TermsOfUse.txt

#include "TorusGames-Common.h"
#include "TorusGames-OpenGL.h"
#include <stddef.h>	//	for offsetof()


//	The rectangle Vertex Buffer Object (VBO) will contain
//	the following data for each of its vertices.
typedef struct
{
	float	pos[3],	//	position (x,y,0), where the third coordinate (always 0 in TorusGamesRectangle)
					//		lets TorusGamesRectangleWithEndcaps pass an "endcap coordinate"
			tex[2];	//	texture coordinates (u,v)
} RectangleVBOData;


static void	BindRectangleVBO(unsigned int aVertexBufferName, unsigned int anIndexBufferName, bool aShadersFlag);


void MakeRectangleVAO(
	unsigned int	aVertexArrayName,
	unsigned int	aVertexBufferName,
	unsigned int	anIndexBufferName,	//	ignored
	unsigned int	anOpenGLVersion)
{
	static const RectangleVBOData	theVertices[4] =
									{
										{{-0.5, -0.5, 0.0}, {0.0, 0.0}},
										{{+0.5, -0.5, 0.0}, {1.0, 0.0}},
										{{-0.5, +0.5, 0.0}, {0.0, 1.0}},
										{{+0.5, +0.5, 0.0}, {1.0, 1.0}}
									};

	if (RUNNING_OPENGL_3)
		;	//	uncomment the following two lines:
//	if (anOpenGLVersion >= VERSION_NUMBER(3, 0))
//		glBindVertexArray(0);

	glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferName);
	glBufferData(GL_ARRAY_BUFFER,
					4 * sizeof(RectangleVBOData),
					theVertices,
					GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	if (RUNNING_OPENGL_3)
		;	//	uncomment the following several lines:
//	if (anOpenGLVersion >= VERSION_NUMBER(3, 0))
//	{
//		glBindVertexArray(aVertexArrayName);
//		BindRectangleVBO(aVertexBufferName, anIndexBufferName);
//		glBindVertexArray(0);
//	}
UNUSED_PARAMETER(anIndexBufferName);	//	REMOVE THIS AFTER UNCOMMENTING ABOVE LINES.
}


void BindRectangleVAO(
	unsigned int	aVertexArrayName,
	unsigned int	aVertexBufferName,
	unsigned int	anIndexBufferName,	//	ignored
	unsigned int	anOpenGLVersion,
	bool			aShadersFlag)	//	REMOVE THIS PARAMETER WHEN REMOVING OPENGL 1.X SUPPORT FROM DESKTOP VERSION.
									//	HMMM... OR MAYBE NOT -- IT SEEMS TO BE NEEDED FOR GLES 2 SUPPORT.
{
	//	When OpenGL 2.1 support eventually gets removed,
	//	this function may be eliminated and the caller may instead
	//	call glBindVertexArray(aVertexArrayName) directly.

	if (RUNNING_OPENGL_3)
		;	//	uncomment the following several lines:
//	if (anOpenGLVersion >= VERSION_NUMBER(3, 0))
//		glBindVertexArray(aVertexArrayName);
//	else
		BindRectangleVBO(aVertexBufferName, anIndexBufferName, aShadersFlag);
}

static void BindRectangleVBO(
	unsigned int	aVertexBufferName,
	unsigned int	anIndexBufferName,	//	ignored
	bool			aShadersFlag)	//	REMOVE THIS PARAMETER WHEN REMOVING OPENGL 1.X SUPPORT FROM DESKTOP VERSION.
									//	HMMM... OR MAYBE NOT -- IT SEEMS TO BE NEEDED FOR GLES 2 SUPPORT.
{
	//	Under OpenGL 2.1 or earlier, this function 
	//		gets called once per frame, from BindRectangleVAO().
	//	Under OpenGL 3.0 or later, this function
	//		gets called only once, from MakeRectangleVAO().
	//	When OpenGL 2.1 support eventually gets removed,
	//		this function may be eliminated and its contents
	//		may be moved directly into MakeRectangleVAO().

	glBindBuffer(GL_ARRAY_BUFFER, aVertexBufferName);

	if (aShadersFlag)
	{
#if defined(SUPPORT_OPENGL_ES_2) || ! defined(TORUS_GAMES_FOR_OPENGL_ES)
		glEnableVertexAttribArray(ATTRIBUTE_POSITION);
		glVertexAttribPointer(ATTRIBUTE_POSITION,  3, GL_FLOAT, GL_FALSE, sizeof(RectangleVBOData), (void *)offsetof(RectangleVBOData, pos));

		glEnableVertexAttribArray(ATTRIBUTE_TEX_COORD);
		glVertexAttribPointer(ATTRIBUTE_TEX_COORD, 2, GL_FLOAT, GL_FALSE, sizeof(RectangleVBOData), (void *)offsetof(RectangleVBOData, tex));
#endif
	}
	else
	{
#if defined(SUPPORT_OPENGL_ES_1) || ! defined(TORUS_GAMES_FOR_OPENGL_ES)
		glVertexPointer  (3, GL_FLOAT, sizeof(RectangleVBOData), (void *)offsetof(RectangleVBOData, pos));
		glTexCoordPointer(2, GL_FLOAT, sizeof(RectangleVBOData), (void *)offsetof(RectangleVBOData, tex));
#endif
	}

	UNUSED_PARAMETER(anIndexBufferName);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

void DrawRectangleVAO(void)
{
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
}
