//	TorusGamesSample.c
//
//	Code specific to the Sample game.
//	This code knows about OpenGL, but does not
//	know about Windows or any other operating system.
//
//	THIS "SAMPLE GAME" ISN'T A GAME AT ALL, BUT MERELY
//	A COLLECTION OF EMPTY FUNCTIONS.  USE IT AS A STARTING POINT
//	FOR IMPLEMENTING REAL TORUS GAMES.
//
//	© 2010 by Jeff Weeks
//	See TermsOfUse.txt

#include "TorusGames-Common.h"
#include "TorusGames-OpenGL.h"


//	< Declare SampleSetUp() in TorusGames-Common.h >
extern void			SampleSetUp(ModelData *md);

//	Public functions
static void	SampleShutDown(ModelData *md);
static void	SampleSetUpTextures(ModelData *md, ErrorText *aFirstError);
static void	SampleShutDownTextures(ModelData *md);
static void	SampleReset(ModelData *md);
static bool	SampleDragBegin(ModelData *md, bool aShiftOrControlKeyIsDown);
static void	SampleDragObject(ModelData *md, double aHandLocalDeltaH, double aHandLocalDeltaV);
static void	SampleDragEnd(ModelData *md);
static void	SampleSimulationUpdate(ModelData *md);
static void	SampleDraw(ModelData *md);

//	Private functions
//	<ALL OTHER STATIC FUNCTIONS GET DECLARED HERE>


void SampleSetUp(ModelData *md)
{
	//	Initialize function pointers.
	md->itsGameShutDown			= &SampleShutDown;
	md->itsGameSetUpTextures	= &SampleSetUpTextures;
	md->itsGameShutDownTextures	= &SampleShutDownTextures;
	md->itsGameReset			= &SampleReset;
	md->itsGameHandMoved		= NULL;
	md->itsGameDragBegin		= &SampleDragBegin;
	md->itsGameDragObject		= &SampleDragObject;
	md->itsGameDragEnd			= &SampleDragEnd;
	md->itsGameCharacterInput	= NULL;
	md->itsGameSpecialCommand	= NULL;
	md->itsGameSimulationUpdate	= &SampleSimulationUpdate;
	md->itsGameDraw				= &SampleDraw;

	//	Set up the game.
	SampleReset(md);
}


static void SampleShutDown(ModelData *md)
{
	//	The caller must ensure that the OpenGL context is set
	//	so textures may be released.
	SampleShutDownTextures(md);

	//	Free private memory, terminate private threads, etc.
}


static void SampleSetUpTextures(
	ModelData	*md,
	ErrorText	*aFirstError)
{
	UNUSED_PARAMETER(md);
	UNUSED_PARAMETER(aFirstError);

//	SetUpOneTexture(md, &md->itsGameOf.Sample.itsTextureNameSomething,	"Sample/Something.rgba",	TextureWrap,	TextureRGBA,	aFirstError);
}


static void SampleShutDownTextures(ModelData *md)
{
	UNUSED_PARAMETER(md);

//	ShutdownOneTexture(&md->itsGameOf.Sample.itsTextureNameSomething);
}


static void SampleReset(ModelData *md)
{
	//	Abort any pending simulation.
	//	(Free resources if necessary.)
	SimulationEnd(md);
}


static bool SampleDragBegin(
	ModelData	*md,
	bool		aShiftOrControlKeyIsDown)
{
	UNUSED_PARAMETER(md);
	UNUSED_PARAMETER(aShiftOrControlKeyIsDown);

	//	Set itsDragIsReversed only if SampleDragObject() will need it.
	md->itsDragIsReversed = false;

	return false;
}


static void SampleDragObject(
	ModelData	*md,
	double		aHandLocalDeltaH,
	double		aHandLocalDeltaV)
{
	UNUSED_PARAMETER(md);
	UNUSED_PARAMETER(aHandLocalDeltaH);
	UNUSED_PARAMETER(aHandLocalDeltaV);
}


static void SampleDragEnd(ModelData *md)
{
	UNUSED_PARAMETER(md);
}


static void SampleSimulationUpdate(ModelData *md)
{
	UNUSED_PARAMETER(md);
}


static void SampleDraw(ModelData *md)
{
	//	Draw the background.
//	DrawFundamentalDomain(md, md->itsGameOf.Sample.itsTextureNameSomething, 1);
	
	//	Draw glide reflection axes if appropriate.
	DrawKleinAxes(	md,
					PREMULTIPLY_RGBA(1.0, 0.0, 0.0, 1.0),
					PREMULTIPLY_RGBA(0.5, 1.0, 1.0, 1.0) );
}
