// 	TorusGamesHelpController.m
//
//	© 2010 by Jeff Weeks
//	See TermsOfUse.txt

#import "TorusGamesHelpController.h"
#import "TorusGames-Common.h"	//	for BUFFER_LENGTH
#import "TorusGamesPodOSUtilities.h"
#import "TorusGamesWebViewController.h"


struct
{
	char		*itsName;
	NSString	*itsDirectoryName,
				*itsFileName;
	bool		itsFileIsLocalized;
	NSString	*itsImageName;
} gHelpInfo[2][4] =
{
	{
		{"How to Play",			@"Help",	@"HowToPlay-%c%c.html",				true,	@"Table Images/Help-HowToPlay.png"			},
		{"Wraparound Universe",	@"Help",	@"WraparoundUniverse-%c%c.html",	true,	@"Table Images/Help-WraparoundUniverse.png"	},
		{"Practice Board",		nil,		nil,								false,	@"Table Images/Help-PracticeBoard.png"		},
		{"Contact",				@"Help",	@"Contact-%c%c.html",				true,	@"Table Images/Help-Contact.png"			}
	},
	{
		{"Translators",			@"HTML",	@"Translators.html",				false,	@"Table Images/Thanks-Translators.png"		},
		{"Math for America",	@"HTML",	@"MathForAmerica.html",				false,	@"Table Images/Thanks-MathForAmerica.png"	},
		{"UNUSED",				nil,		nil,								false,	nil											},
		{"UNUSED",				nil,		nil,								false,	nil											}
	}
};


@implementation TorusGamesHelpController

- (id)initWithParent:	(TorusGamesTabBarController *)aParent
		gameChoice:		(GameType *)aGameChoice
		filler:			(unsigned int)aDummyValue	//	See comment in TorusGamesHelpController.h
{
	if ((self = [super initWithTag:TabPageHelp parent:aParent]) != nil)
	{
		itsGameChoice = aGameChoice;
	}
	return self;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)aTableView
{
	return 2;
}

- (NSString *)tableView:(UITableView *)aTableView titleForHeaderInSection:(NSInteger)aSection
{
	switch (aSection)
	{
		case 0:		return GetLocalizedTextAsNSString("Help"  );
		case 1:		return GetLocalizedTextAsNSString("Thanks");
		default:	return @"?";
	}
}

- (NSInteger)tableView:(UITableView *)aTableView numberOfRowsInSection:(NSInteger)aSection
{
	switch (aSection)
	{
		case 0:		return 4;
		case 1:		return 2;
		default:	return 0;
	}
}

- (UITableViewCell *)tableView:(UITableView *)aTableView cellForRowAtIndexPath:(NSIndexPath *)anIndexPath
{
	UITableViewCell	*theCell;
	UILabel			*theLabel;
	unsigned int	theSection,
					theRow;
	bool			theCheckmark;

	theCell			= [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:nil] autorelease];
	theLabel		= [theCell textLabel];
	theSection		= [anIndexPath section];
	theRow			= [anIndexPath row];

	[theLabel setText:GetLocalizedTextAsNSString(gHelpInfo[theSection][theRow].itsName)];
	if (gHelpInfo[theSection][theRow].itsFileName == nil)
		theCheckmark = (*itsGameChoice == GameIntro);
	else
		theCheckmark = false;
	[theCell setAccessoryType:(theCheckmark ? UITableViewCellAccessoryCheckmark : UITableViewCellAccessoryNone)];
	[[theCell imageView] setImage:[UIImage imageNamed:gHelpInfo[theSection][theRow].itsImageName]];

	return theCell;
}

- (NSIndexPath *)tableView:(UITableView *)aTableView willSelectRowAtIndexPath:(NSIndexPath *)anIndexPath
{
	unsigned int				theSection,
								theRow;
	NSString					*theDirectoryName,
								*theFileName;
	TorusGamesWebViewController	*theWebViewController;

	theSection			= [anIndexPath section];
	theRow				= [anIndexPath row];
	theDirectoryName	= gHelpInfo[theSection][theRow].itsDirectoryName;
	theFileName			= gHelpInfo[theSection][theRow].itsFileName;

	if (theFileName != nil)	//	request for HTML page
	{
		if (gHelpInfo[theSection][theRow].itsFileIsLocalized)
			theFileName = [NSString stringWithFormat:theFileName, gLanguageCode[0], gLanguageCode[1]];

		theWebViewController = [[TorusGamesWebViewController alloc]
			initWithDirectory:theDirectoryName page:theFileName];
		[theWebViewController setModalTransitionStyle:UIModalTransitionStyleCoverVertical];
		[self presentModalViewController:theWebViewController animated:YES];
		[theWebViewController release];
	}
	else
	{
		//	The user has pressed the "Practice Board" item.
		*itsGameChoice = GameIntro;
		[self buttonWasPressed];
	}

	return nil;
}

@end
