// 	TorusGamesKeyboardView.m
//
//	© 2010 by Jeff Weeks
//	See TermsOfUse.txt

#import "TorusGamesKeyboardView.h"
#import "TorusGamesMainViewController.h"
#import "TorusGamesModelData.h"
#import "TorusGames-Common.h"


#define	KOREAN_SHIFT_CHARACTER	L'⇧'
#define KOREAN_ENTER_CHARACTER	L'↵'
#define JAMO_KEY				0xFFFFFFFC
#define KOREAN_SHIFT_KEY		0xFFFFFFFD
#define KOREAN_ENTER_KEY		0xFFFFFFFE
#define GENERIC_KEY				0xFFFFFFFF


typedef struct
{
	unsigned int	itsNumRows,
					*itsRowLengths;
	float			itsFullWidth,	//	measured in key-widths (ignoring key insets)
					*itsRowStarts;	//	measured in key-widths (ignoring key insets)
	unichar			**itsRows;
} KeyboardLayout;


static KeyboardLayout
	gKeyboardDE =
	{
		3,
		(unsigned int [3]) {10, 9, 7},
		10.0,
		(float [3]) {0.0, 0.5, 1.5},
		(unichar *[3])
		{
			(unichar [10]) {L'q', L'w', L'e', L'r', L't', L'z', L'u', L'i', L'o', L'p'},
			(unichar [ 9]) {L'a', L's', L'd', L'f', L'g', L'h', L'j', L'k', L'l'},
			(unichar [ 7]) {L'y', L'x', L'c', L'v', L'b', L'n', L'm'}
		}
	},
	gKeyboardEN =
	{
		3,
		(unsigned int [3]) {10, 9, 7},
		10.0,
		(float [3]) {0.0, 0.5, 1.5},
		(unichar *[3])
		{
			(unichar [10]) {L'q', L'w', L'e', L'r', L't', L'y', L'u', L'i', L'o', L'p'},
			(unichar [ 9]) {L'a', L's', L'd', L'f', L'g', L'h', L'j', L'k', L'l'},
			(unichar [ 7]) {L'z', L'x', L'c', L'v', L'b', L'n', L'm'}
		}
	},
	gKeyboardES =
	{
		3,
		(unsigned int [3]) {10, 9, 7},
		10.0,
		(float [3]) {0.0, 0.5, 1.5},
		(unichar *[3])
		{
			(unichar [10]) {L'q', L'w', L'e', L'r', L't', L'y', L'u', L'i', L'o', L'p'},
			(unichar [ 9]) {L'a', L's', L'd', L'f', L'g', L'h', L'j', L'k', L'l'},
			(unichar [ 7]) {L'z', L'x', L'c', L'v', L'b', L'n', L'm'}
		}
	},
	gKeyboardFR =
	{
		3,
		(unsigned int [3]) {10, 10, 6},
		10.0,
		(float [3]) {0.0, 0.0, 2.0},
		(unichar *[3])
		{
			(unichar [10]) {L'a', L'z', L'e', L'r', L't', L'y', L'u', L'i', L'o', L'p'},
			(unichar [10]) {L'q', L's', L'd', L'f', L'g', L'h', L'j', L'k', L'l', L'm'},
			(unichar [ 6]) {L'w', L'x', L'c', L'v', L'b', L'n'}
		}
	},
	gKeyboardIT =
	{
		3,
		(unsigned int [3]) {10, 9, 7},
		10.0,
		(float [3]) {0.0, 0.5, 1.5},
		(unichar *[3])
		{
			(unichar [10]) {L'q', L'w', L'e', L'r', L't', L'y', L'u', L'i', L'o', L'p'},
			(unichar [ 9]) {L'a', L's', L'd', L'f', L'g', L'h', L'j', L'k', L'l'},
			(unichar [ 7]) {L'z', L'x', L'c', L'v', L'b', L'n', L'm'}
		}
	},
	gKeyboardJA =
	{
		3,
		(unsigned int [3]) {5, 8, 8},
		10.0,
		(float [3]) {0.5, 1.0, 1.0},
		(unichar *[3])
		{
			(unichar [ 5]) {L'あ', L'い', L'う', L'え', L'お'},
			(unichar [ 8]) {L'あ', L'か', L'が', L'さ', L'ざ', L'た', L'だ', L'な'},
			(unichar [ 8]) {L'は', L'ば', L'ぱ', L'ま', L'や', L'ら', L'わ', L'ん'}
		}
	},
	gKeyboardKO =
	{
		3,
		(unsigned int [3]) {10, 10, 8},
		10.0,
		(float [3]) {0.0, 0.0, 1.0},
		(unichar *[3])
		{
			(unichar [10]) {L'ㅂ', L'ㅈ', L'ㄷ', L'ㄱ', L'ㅅ', L'ㅛ', L'ㅕ', L'ㅑ', L'ㅐ', L'ㅔ'},
			(unichar [10]) {L'ㅁ', L'ㄴ', L'ㅇ', L'ㄹ', L'ㅎ', L'ㅗ', L'ㅓ', L'ㅏ', L'ㅣ', KOREAN_ENTER_CHARACTER},
			(unichar [ 8]) {L'ㅋ', L'ㅌ', L'ㅊ', L'ㅍ', L'ㅠ', L'ㅜ', L'ㅡ', KOREAN_SHIFT_CHARACTER}
		}
	},
	gKeyboardNL =
	{
		3,
		(unsigned int [3]) {10, 9, 7},
		10.0,
		(float [3]) {0.0, 0.5, 1.5},
		(unichar *[3])
		{
			(unichar [10]) {L'q', L'w', L'e', L'r', L't', L'y', L'u', L'i', L'o', L'p'},
			(unichar [ 9]) {L'a', L's', L'd', L'f', L'g', L'h', L'j', L'k', L'l'},
			(unichar [ 7]) {L'z', L'x', L'c', L'v', L'b', L'n', L'm'}
		}
	},
	gKeyboardPT =
	{
		3,
		(unsigned int [3]) {10, 9, 7},
		10.0,
		(float [3]) {0.0, 0.5, 1.5},
		(unichar *[3])
		{
			(unichar [10]) {L'q', L'w', L'e', L'r', L't', L'y', L'u', L'i', L'o', L'p'},
			(unichar [ 9]) {L'a', L's', L'd', L'f', L'g', L'h', L'j', L'k', L'l'},
			(unichar [ 7]) {L'z', L'x', L'c', L'v', L'b', L'n', L'm'}
		}
	},
	gKeyboardRU =
	{
		3,
		(unsigned int [3]) {11, 11, 9},
		11.0,
		(float [3]) {0.0, 0.0, 1.0},
		(unichar *[3])
		{
			(unichar [11]) {L'й', L'ц', L'у', L'к', L'е', L'н', L'г', L'ш', L'щ', L'з', L'х'},
			(unichar [11]) {L'ф', L'ы', L'в', L'а', L'п', L'р', L'о', L'л', L'д', L'ж', L'э'},
			(unichar [ 9]) {L'я', L'ч', L'с', L'м', L'и', L'т', L'ь', L'б', L'ю'}
		}
	},
	gKeyboardVI =
	{
		3,
		(unsigned int [3]) {10, 9, 7},
		10.0,
		(float [3]) {0.0, 0.5, 1.5},
		(unichar *[3])
		{
			(unichar [10]) {L'q', L'w', L'e', L'r', L't', L'y', L'u', L'i', L'o', L'p'},
			(unichar [ 9]) {L'a', L's', L'd', L'f', L'g', L'h', L'j', L'k', L'l'},
			(unichar [ 7]) {L'z', L'x', L'c', L'v', L'b', L'n', L'm'}
		}
	};


@implementation TorusGamesKeyboardView

- (id)initWithController:(TorusGamesMainViewController *)aController modelData:(TorusGamesModelData *)someModelData
{
	KeyboardLayout	*theKeyboard;
	UIColor			*theColor,		//	for kana selector keys and all non-Japanese keys
					*theColorVar;	//	for the 5 variable kana keys
	unichar			theCharacter;
	UIButton		*theKey;
	CGFloat			theKeyWidth,
					theKeyHeight;
	unsigned int	theRow,
					theCol;
	CGFloat			theKeyWidth2;
	
	if ((self = [super initWithFrame:
		(CGRect){{KEYBOARD_MARGIN_H, SCREEN_HEIGHT - KEYBOARD_HEIGHT - MIDDLE_ZONE_MARGIN/2}, {KEYBOARD_WIDTH, KEYBOARD_HEIGHT}}]) != nil)
	{
		itsController	= aController;
		itsModelData	= someModelData;
		for (theRow = 0; theRow < MAX_ROWS; theRow++)
			for (theCol = 0; theCol < MAX_COLUMNS; theCol++)
				itsKeys[theRow][theCol] = nil;
		itsKoreanKeyboardIsUpshifted = false;

			 if (SAME_LANGUAGE_CODE(gLanguageCode, "de")) theKeyboard = &gKeyboardDE;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "en")) theKeyboard = &gKeyboardEN;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "es")) theKeyboard = &gKeyboardES;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "fr")) theKeyboard = &gKeyboardFR;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "it")) theKeyboard = &gKeyboardIT;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "ja")) theKeyboard = &gKeyboardJA;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "ko")) theKeyboard = &gKeyboardKO;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "nl")) theKeyboard = &gKeyboardNL;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "pt")) theKeyboard = &gKeyboardPT;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "ru")) theKeyboard = &gKeyboardRU;
		else if (SAME_LANGUAGE_CODE(gLanguageCode, "vi")) theKeyboard = &gKeyboardVI;
		else theKeyboard = &gKeyboardEN;
		
		//	Make sure theKeyboard doesn't overrun the fixed space allowed for itsKeys.
		if (theKeyboard->itsNumRows > MAX_ROWS)
			return nil;
		for (theRow = 0; theRow < theKeyboard->itsNumRows; theRow++)
			if (theKeyboard->itsRowLengths[theRow] > MAX_COLUMNS)
				return nil;

		theColor	= [UIColor colorWithRed:0.00 green:0.00 blue:0.50 alpha:1.00];
		theColorVar	= [UIColor colorWithRed:1.00 green:0.00 blue:0.00 alpha:1.00];

		[self setOpaque:NO];

		theKeyWidth  = KEYBOARD_WIDTH  / theKeyboard->itsFullWidth;
		theKeyHeight = KEYBOARD_HEIGHT / theKeyboard->itsNumRows;
		
		for (theRow = 0; theRow < theKeyboard->itsNumRows; theRow++)
		{
			for (theCol = 0; theCol < theKeyboard->itsRowLengths[theRow]; theCol++)
			{
				theCharacter	= theKeyboard->itsRows[theRow][theCol];
				theKey			= [UIButton buttonWithType:UIButtonTypeRoundedRect];

				itsKeys[theRow][theCol] = theKey;

				if (SAME_LANGUAGE_CODE(gLanguageCode, "ja"))
				{
					if (theRow == 0)	//	variable kana
					{
						//	Record the column index, so we can 
						//	recognize theKey in touchUpInCrosswordKey.
						[theKey setTag:theCol];
					}
					else
					{
						//	Copy a selector kana into the tag, for convenience
						//	in touchUpInCrosswordKey.  This usage won't conflict
						//	with the column indices written immediately above,
						//	because the column indices run only from 0 to 4.
						//	The only non-selector kana in the list is ん,
						//	which is treated as a normal letter.
						if (theCharacter != L'ん')
							[theKey setTag:theCharacter];
						else
							[theKey setTag:GENERIC_KEY];
					}
				}
				else
				if (SAME_LANGUAGE_CODE(gLanguageCode, "ko"))
				{
					if (theCharacter == KOREAN_SHIFT_CHARACTER)
						[theKey setTag:KOREAN_SHIFT_KEY];
					else
					if (theCharacter == KOREAN_ENTER_CHARACTER)
						[theKey setTag:KOREAN_ENTER_KEY];
					else
						[theKey setTag:JAMO_KEY];
				}
				else
				{
					//	We won't need the tag, so set it to something harmless.
					[theKey setTag:GENERIC_KEY];
				}

				[theKey setTitle:		[NSString stringWithCharacters:&theCharacter length:1]
						forState:		UIControlStateNormal];

				[theKey setTitleColor:	(SAME_LANGUAGE_CODE(gLanguageCode, "ja") && theRow == 0)
											? theColorVar : theColor
						forState:		UIControlStateNormal];

				if (SAME_LANGUAGE_CODE(gLanguageCode, "ja") && theRow == 0)
					theKeyWidth2 = (9.0/5.0) * theKeyWidth;	//	5 wide keys fill the space of 9 normal ones.
				else
					theKeyWidth2 = theKeyWidth;

				[theKey setFrame:(CGRect)
					{
						{
							theKeyWidth*theKeyboard->itsRowStarts[theRow] + theKeyWidth2*theCol + KEYINSET_H,
							theKeyHeight*theRow + KEYINSET_V
						},
						{
							theKeyWidth2 - 2*KEYINSET_H,
							theKeyHeight - 2*KEYINSET_V
						}
					}];

				//	Use a slightly larger font for the variable kana keys.
				//	Note:  UIFont caches fonts, so once it's created the larger font,
				//	we'll keep getting pointers to the same object.
				//	Thus, there's no need for us to cache a pointer to it ourselves.
				if (SAME_LANGUAGE_CODE(gLanguageCode, "ja") && theRow == 0)
					[[theKey titleLabel] setFont:[[[theKey titleLabel] font] fontWithSize:
						[[[theKey titleLabel] font] pointSize] + 2.0]];

				[[theKey titleLabel] setTextAlignment:UITextAlignmentCenter];

				[theKey
					addTarget:			self
					action:				@selector(touchUpInCrosswordKey:)
					forControlEvents:	UIControlEventTouchUpInside];

				[self addSubview:theKey];
			}
		}
	}
	return self;
}


- (void)touchUpInCrosswordKey:(id)sender
{
	UIButton		*theButton;
	unsigned int	theButtonTag;
	NSString		*theButtonTitle;
	unichar			theCharacter,
					*theVariants;
	unsigned int	theCol;
	
	theButton		= (UIButton *) sender;
	theButtonTag	= [theButton tag];
	theButtonTitle	= [theButton currentTitle];
	if (theButtonTitle != nil && [theButtonTitle length] > 0)
		theCharacter = [theButtonTitle characterAtIndex:0];
	else
		theCharacter = 0;	//	should never occur
	
	if (theButtonTag < 5)
	{
		//	The user has pressed one of the five variable kana key in a Japanese puzzle.

		//	Ignore non-existent or obsolete kana (yi, ye, wi, wu, we),
		//	which get displayed to the user as full-width spaces,
		//	but process all other variable kana normally.
		if (theCharacter == L'　')
			theCharacter = 0;
	}
	else
	if (theButtonTag == JAMO_KEY
	 || theButtonTag == KOREAN_SHIFT_KEY
	 || theButtonTag == KOREAN_ENTER_KEY)
	{
		//	Only the shift key will up-shift the letters.
		if ( ! itsKoreanKeyboardIsUpshifted		//	If the down-shifted key are currently active
		 && theButtonTag == KOREAN_SHIFT_KEY)	//	and the user pressed the shift key, then
		{
			//	Insert up-shifted values into the top row.
			[itsKeys[0][0] setTitle:@"ㅃ" forState:UIControlStateNormal];
			[itsKeys[0][1] setTitle:@"ㅉ" forState:UIControlStateNormal];
			[itsKeys[0][2] setTitle:@"ㄸ" forState:UIControlStateNormal];
			[itsKeys[0][3] setTitle:@"ㄲ" forState:UIControlStateNormal];
			[itsKeys[0][4] setTitle:@"ㅆ" forState:UIControlStateNormal];
			[itsKeys[0][8] setTitle:@"ㅒ" forState:UIControlStateNormal];
			[itsKeys[0][9] setTitle:@"ㅖ" forState:UIControlStateNormal];
			
			itsKoreanKeyboardIsUpshifted = true;
		}
		else	//	Any key (Jamo or shift key) may down-shift the letters.
		{
			if (itsKoreanKeyboardIsUpshifted)
			{
				//	Insert down-shifted values into the top row.
				[itsKeys[0][0] setTitle:@"ㅂ" forState:UIControlStateNormal];
				[itsKeys[0][1] setTitle:@"ㅈ" forState:UIControlStateNormal];
				[itsKeys[0][2] setTitle:@"ㄷ" forState:UIControlStateNormal];
				[itsKeys[0][3] setTitle:@"ㄱ" forState:UIControlStateNormal];
				[itsKeys[0][4] setTitle:@"ㅅ" forState:UIControlStateNormal];
				[itsKeys[0][8] setTitle:@"ㅐ" forState:UIControlStateNormal];
				[itsKeys[0][9] setTitle:@"ㅔ" forState:UIControlStateNormal];

				itsKoreanKeyboardIsUpshifted = false;
			}
		}
		
		//	Don't send the shift key itself to the puzzle.
		if (theButtonTag == KOREAN_SHIFT_KEY)
			theCharacter = 0;
		
		//	Let the enter key send a newline.
		if (theButtonTag == KOREAN_ENTER_KEY)
			theCharacter = L'\n';
	}
	else
	if (theButtonTag < GENERIC_KEY)
	{
		//	The user has pressed one of the 15 selector keys in a Japanese puzzle.
		//	Set the five variable kana accordingly.
		switch (theButtonTag)
		{
			case L'あ': theVariants = (unichar [5]) {L'あ', L'い', L'う', L'え', L'お'}; break;
			case L'か': theVariants = (unichar [5]) {L'か', L'き', L'く', L'け', L'こ'}; break;
			case L'が': theVariants = (unichar [5]) {L'が', L'ぎ', L'ぐ', L'げ', L'ご'}; break;
			case L'さ': theVariants = (unichar [5]) {L'さ', L'し', L'す', L'せ', L'そ'}; break;
			case L'ざ': theVariants = (unichar [5]) {L'ざ', L'じ', L'ず', L'ぜ', L'ぞ'}; break;
			case L'た': theVariants = (unichar [5]) {L'た', L'ち', L'つ', L'て', L'と'}; break;
			case L'だ': theVariants = (unichar [5]) {L'だ', L'ぢ', L'づ', L'で', L'ど'}; break;
			case L'な': theVariants = (unichar [5]) {L'な', L'に', L'ぬ', L'ね', L'の'}; break;
			case L'は': theVariants = (unichar [5]) {L'は', L'ひ', L'ふ', L'へ', L'ほ'}; break;
			case L'ば': theVariants = (unichar [5]) {L'ば', L'び', L'ぶ', L'べ', L'ぼ'}; break;
			case L'ぱ': theVariants = (unichar [5]) {L'ぱ', L'ぴ', L'ぷ', L'ぺ', L'ぽ'}; break;
			case L'ま': theVariants = (unichar [5]) {L'ま', L'み', L'む', L'め', L'も'}; break;
			case L'や': theVariants = (unichar [5]) {L'や', L'　', L'ゆ', L'　', L'よ'}; break;
			case L'ら': theVariants = (unichar [5]) {L'ら', L'り', L'る', L'れ', L'ろ'}; break;
			case L'わ': theVariants = (unichar [5]) {L'わ', L'　', L'　', L'　', L'を'}; break;
			default:   theVariants = (unichar [5]) {L'？', L'？', L'？', L'？', L'？'}; break;
		}
		for (theCol = 0; theCol < 5; theCol++)
		{
			[itsKeys[0][theCol] setTitle:
				[NSString stringWithCharacters:&theVariants[theCol] length:1]
				forState:UIControlStateNormal];
		}
		
		//	Don't send the selector itself to the puzzle.
		theCharacter = 0;
	}
	else
	{
		//	The user has pressed a key in a non-Japanese puzzle.
		//	Process theCharacter normally.
	}

	if (theCharacter != 0)
	{
		//	Ignore the keytaps after the game is over.
		//
		//	In particular, we want to avoid the situation that the user
		//	1.	taps a random key (perhaps by accident), and then
		//	2.	resets the game.
		//	The game reset would quickly hide the toolbar,
		//	but the random keytap's call to showToolbarWithAnimation
		//	would arrive after a 4-second delay and show the toolbar again,
		//	with no easy way for the user to get rid of it.
		//	By ignoring keytaps after the game is over,
		//	we avoid that situation.
		if ( ! [itsModelData modelData]->itsGameIsOver )
		{
			//	CharacterInput() will ignore characters typed after the game is over.
			CharacterInput([itsModelData modelData], theCharacter);

			//	Don't let SoundCrosswordKeypress compete with SoundCrosswordComplete.
			if ( ! [itsModelData modelData]->itsGameIsOver )	
				PlayTheSound(SoundCrosswordKeypress);
			
			//	If the user has just completed the puzzle,
			//	wait a few seconds to let the victory sound finish playing
			//	and also to let the user enjoy the satisfaction,
			//	then show the Toolbar.  The toolbar will remain visible
			//	until the user taps the Reset key to advance to the next puzzle.
			if ([itsModelData modelData]->itsGameIsOver)
				[itsController	performSelector:	@selector(showToolbarWithAnimation)
								withObject:			nil
								afterDelay:			4.0];
		}
	}
}


@end
